package lib

import (
	"bytes"
	"fmt"
	"math"
	"math/rand/v2"
	"net/http"
	"sync"
	"sync/atomic"
	"time"
)

var client = &http.Client{}

func (e *RequestError) Error() string {
	return fmt.Sprintf("error making %s request to %s: %v", e.Verb, e.URL, e.Err)
}

func makeRequest(verb, url, token string, jsonData []byte, second int) error {
	startTime := time.Now()

	var req *http.Request
	var err error
	if jsonData != nil {
		req, err = http.NewRequest(verb, url, bytes.NewBuffer(jsonData))
		req.Header.Set("Content-Type", "application/json")
	} else {
		req, err = http.NewRequest(verb, url, nil)
	}
	if err != nil {
		return &RequestError{Verb: verb, URL: url, Err: err}
	}

	if token != "" {
		req.Header.Set("Authorization", "Bearer "+token)
	}

	resp, err := client.Do(req)
	if err != nil {
		return &RequestError{Verb: verb, URL: url, Err: err}
	}
	defer resp.Body.Close()

	duration := time.Since(startTime)

	UpdateMetrics(duration, resp, second)

	return nil
}

func SendRequests(url string, patterns []RequestPattern, maxRequests int, requestsPerSecond float64, token string, jsonData []byte) {
	metrics = PerformanceMetrics{
		MinLatency:       time.Duration(math.MaxInt64),
		ResponseCounters: make(map[int]int32),
	}

	startTime := time.Now()
	rateLimit := time.Second / time.Duration(requestsPerSecond)
	ticker := time.NewTicker(rateLimit)
	defer ticker.Stop()

	var requestCount int32
	var wg sync.WaitGroup

	patternIndex := 0
	sequenceCount := 0

	for range ticker.C {
		if int(requestCount) >= maxRequests {
			break
		}

		var selectedVerb string
		if patterns[0].Sequence > 0 {
			currentPattern := patterns[patternIndex]
			selectedVerb = currentPattern.Verb

			sequenceCount++
			if sequenceCount >= currentPattern.Sequence {
				sequenceCount = 0
				patternIndex = (patternIndex + 1) % len(patterns)
			}
		} else {
			rand := rand.Float64() * 100
			cumulative := 0.0
			for _, p := range patterns {
				cumulative += p.Percentage
				if rand <= cumulative {
					selectedVerb = p.Verb
					break
				}
			}
		}

		wg.Add(1)
		go func(verb string) {
			defer wg.Done()
			err := makeRequest(verb, url, token, jsonData, int(time.Since(startTime).Seconds()))
			if err != nil {
				fmt.Printf("Error making request: %v\n", err)
			}
			atomic.AddInt32(&requestCount, 1)
		}(selectedVerb)
	}

	wg.Wait()

	time.Sleep(100 * time.Millisecond)

	CalculateAndPrintMetrics(startTime, requestsPerSecond, url, patterns)
}
