package main

import (
	"flag"
	"fmt"
	"loadr/lib"
	"os"
)

var version string = "1.0.2"

func parseCommandLine() (float64, int, string, []lib.RequestPattern, string, string) {
	requestsPerSecond := flag.Float64("rate", 10, "Number of requests per second")
	maxRequests := flag.Int("max", 50, "Maximum number of requests to send (0 for unlimited)")
	url := flag.String("url", "https://example.com", "The URL to make requests to")
	pattern := flag.String("pattern", "", `Request pattern (e.g., "5p" for 5 POSTs, "1p5g" for 1 POST + 5 GETs)`)
	jsonFilePath := flag.String("json", "", "Path to the JSON file with request data")
	bearerToken := flag.String("token", "", "Bearer token for authorization")
	versionFlag := flag.Bool("version", false, "Print the version and exit")
	versionFlagShort := flag.Bool("v", false, "Print the version and exit")

	flag.Parse()

	if *versionFlag || *versionFlagShort {
		fmt.Println("Version:", version)
		os.Exit(0)
	}

	fmt.Printf("Received pattern value: '%s'\n", *pattern)

	patterns, err := parsePattern(*pattern)
	if err != nil {
		fmt.Printf("Warning: %v. Using default GET pattern.\n", err)
		patterns = []lib.RequestPattern{{Verb: "GET", Percentage: 100}}
	} else {
		fmt.Printf("Using pattern: %+v\n", patterns)
	}

	return *requestsPerSecond, *maxRequests, *url, patterns, *jsonFilePath, *bearerToken
}

func parsePattern(pattern string) ([]lib.RequestPattern, error) {
	if pattern == "" {
		return []lib.RequestPattern{{Verb: "GET", Percentage: 100}}, nil
	}

	var patterns []lib.RequestPattern
	var current int

	for i := 0; i < len(pattern); i++ {
		c := pattern[i]
		switch {
		case c >= '0' && c <= '9':
			current = current*10 + int(c-'0')
		case c == 'p' || c == 'P':
			if current == 0 {
				current = 1
			}
			patterns = append(patterns, lib.RequestPattern{
				Verb:       "POST",
				Sequence:   current,
				Percentage: 0,
			})
			current = 0
		case c == 'g' || c == 'G':
			if current == 0 {
				current = 1
			}
			patterns = append(patterns, lib.RequestPattern{
				Verb:       "GET",
				Sequence:   current,
				Percentage: 0,
			})
			current = 0
		default:
			return nil, fmt.Errorf("invalid pattern character: %c", c)
		}
	}

	if len(patterns) == 0 {
		return nil, fmt.Errorf("no valid patterns found in: %s", pattern)
	}

	total := 0
	for _, p := range patterns {
		total += p.Sequence
	}
	for i := range patterns {
		patterns[i].Percentage = float64(patterns[i].Sequence) / float64(total) * 100
	}

	return patterns, nil
}

func readJSONFile(filePath string) ([]byte, error) {
	if filePath == "" {
		return nil, nil
	}
	return os.ReadFile(filePath)
}

func main() {
	requestsPerSecond, maxRequests, url, patterns, jsonFilePath, bearerToken := parseCommandLine()

	if maxRequests <= 0 {
		fmt.Println("Error: max must be an integer greater than 0")
		return
	}

	jsonData, err := readJSONFile(jsonFilePath)
	if err != nil {
		fmt.Println("Error reading JSON file:", err)
		return
	}

	lib.SendRequests(url, patterns, maxRequests, requestsPerSecond, bearerToken, jsonData)
}
